@@ -112,6 +112,11 @@ struct volume_group {
 	 * FIXME: Move the next fields into a different struct?
 	 */
 
+	/*
+	 * List of removed logical volumes by _lv_reduce.
+	 */
+	struct dm_list removed_lvs;
+
 	/*
 	 * List of removed physical volumes by pvreduce.
 	 * They have to get cleared on vg_commit.
