@@ -24,8 +24,10 @@ void init_verbose(int level);
 void init_silent(int silent);
 void init_test(int level);
 void init_md_filtering(int level);
+void init_fwraid_filtering(int level);
 void init_pvmove(int level);
 void init_full_scan_done(int level);
+void init_external_device_info_source(unsigned src);
 void init_obtain_device_list_from_udev(int device_list_from_udev);
 void init_trust_cache(int trustcache);
 void init_debug(int level);
@@ -53,9 +55,11 @@ void set_sysfs_dir_path(const char *path);
 
 int test_mode(void);
 int md_filtering(void);
+int fwraid_filtering(void);
 int pvmove_mode(void);
 int full_scan_done(void);
 int obtain_device_list_from_udev(void);
+unsigned external_device_info_source(void);
 int trust_cache(void);
 int verbose_level(void);
 int silent_mode(void);
