@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-# Copyright (C) 2011-2012 Red Hat, Inc. All rights reserved.
+# Copyright (C) 2011-2015 Red Hat, Inc. All rights reserved.
 #
 # This copyrighted material is made available to anyone wishing to use,
 # modify, copy, or redistribute it subject to the terms and conditions
@@ -31,19 +31,21 @@ prepare_clvmd() {
 	fi
 
 	# skip if we don't have our own clvmd...
-	(which clvmd 2>/dev/null | grep "$abs_builddir") || skip
-	# lvs is executed from clvmd - use our version
-	export LVM_BINARY=$(which lvm)
+	if test -z "${installed_testsuite+varset}"; then
+		(which clvmd 2>/dev/null | grep -q "$abs_builddir") || skip
+	fi
 
-	test -e "$DM_DEV_DIR/control" || dmsetup table # create control node
+	test -e "$DM_DEV_DIR/control" || dmsetup table >/dev/null # create control node
 	# skip if singlenode is not compiled in
-	(clvmd --help 2>&1 | grep "Available cluster managers" | grep "singlenode") || skip
+	(clvmd --help 2>&1 | grep "Available cluster managers" | grep -q "singlenode") || skip
 
 #	lvmconf "activation/monitoring = 1"
 	local run_valgrind=
 	test "${LVM_VALGRIND_CLVMD:-0}" -eq 0 || run_valgrind="run_valgrind"
 	rm -f "$CLVMD_PIDFILE"
-	$run_valgrind clvmd -Isinglenode -d 1 -f &
+	echo "<======== Starting CLVMD ========>"
+	# lvs is executed from clvmd - use our version
+	LVM_LOG_FILE_EPOCH=CLVMD LVM_BINARY=$(which lvm) $run_valgrind clvmd -Isinglenode -d 1 -f &
 	echo $! > LOCAL_CLVMD
 
 	for i in $(seq 1 100) ; do
@@ -61,13 +63,14 @@ prepare_dmeventd() {
 	fi
 
 	# skip if we don't have our own dmeventd...
-	(which dmeventd 2>/dev/null | grep "$abs_builddir") || skip
-
+	if test -z "${installed_testsuite+varset}"; then
+		(which dmeventd 2>/dev/null | grep -q "$abs_builddir") || skip
+	fi
 	lvmconf "activation/monitoring = 1"
 
 	local run_valgrind=
 	test "${LVM_VALGRIND_DMEVENTD:-0}" -eq 0 || run_valgrind="run_valgrind"
-	$run_valgrind dmeventd -f "$@" &
+	LVM_LOG_FILE_EPOCH=DMEVENTD $run_valgrind dmeventd -f "$@" &
 	echo $! > LOCAL_DMEVENTD
 
 	# FIXME wait for pipe in /var/run instead
@@ -80,21 +83,25 @@ prepare_dmeventd() {
 }
 
 prepare_lvmetad() {
-	test $# -eq 0 && default_opts="-l wire,debug"
 	rm -f debug.log strace.log
 	# skip if we don't have our own lvmetad...
-	(which lvmetad 2>/dev/null | grep "$abs_builddir") || skip
-
-	lvmconf "global/use_lvmetad = 1"
-	lvmconf "devices/md_component_detection = 0"
+	if test -z "${installed_testsuite+varset}"; then
+		(which lvmetad 2>/dev/null | grep -q "$abs_builddir") || skip
+	fi
 
 	local run_valgrind=
 	test "${LVM_VALGRIND_LVMETAD:-0}" -eq 0 || run_valgrind="run_valgrind"
 
 	kill_sleep_kill_ LOCAL_LVMETAD ${LVM_VALGRIND_LVMETAD:-0}
 
+	# Avoid reconfiguring, if already set to use_lvmetad
+	(grep use_lvmetad CONFIG_VALUES 2>/dev/null | tail -1 | grep -q 1) || \
+		aux lvmconf "global/use_lvmetad = 1" "devices/md_component_detection = 0"
+	# Default debug is "-l all" and could be override
+	# by setting LVM_TEST_LVMETAD_DEBUG_OPTS before calling inittest.
 	echo "preparing lvmetad..."
-	$run_valgrind lvmetad -f "$@" -s "$TESTDIR/lvmetad.socket" $default_opts "$@" &
+	$run_valgrind lvmetad -f "$@" -s "$TESTDIR/lvmetad.socket" \
+		${LVM_TEST_LVMETAD_DEBUG_OPTS--l all} "$@" &
 	echo $! > LOCAL_LVMETAD
 	while ! test -e "$TESTDIR/lvmetad.socket"; do echo -n .; sleep .1; done # wait for the socket
 	echo ok
@@ -123,7 +130,9 @@ lvmetad_dump() {
 
 notify_lvmetad() {
 	if test -e LOCAL_LVMETAD; then
-		pvscan --cache "$@" || true
+		# Ignore results here...
+		LVM_LOG_FILE_EPOCH= pvscan --cache "$@" || true
+		rm -f debug.log
 	fi
 }
 
@@ -133,6 +142,8 @@ teardown_devs_prefixed() {
 	local IFS=$IFS_NL
 	local dm
 
+	rm -rf "$TESTDIR/dev/$prefix"*
+
 	# Resume suspended devices first
 	for dm in $(dm_info suspended,name | grep "^Suspended:.*$prefix"); do
 		echo "dmsetup resume \"${dm#Suspended:}\""
@@ -173,16 +184,16 @@ teardown_devs_prefixed() {
 			num_remaining_devs=$num_devs
 		done
 	fi
+
+	udev_wait
 }
 
 teardown_devs() {
 	# Delete any remaining dm/udev semaphores
 	teardown_udev_cookies
 
-	test -z "$PREFIX" || {
-		rm -rf "$TESTDIR/dev/$PREFIX"*
-		teardown_devs_prefixed "$PREFIX"
-	}
+	test ! -f MD_DEV || cleanup_md_dev
+	test ! -f DEVICES || teardown_devs_prefixed "$PREFIX"
 
 	# NOTE: SCSI_DEBUG_DEV test must come before the LOOP test because
 	# prepare_scsi_debug_dev() also sets LOOP to short-circuit prepare_loop()
@@ -192,22 +203,21 @@ teardown_devs() {
 		test ! -f LOOP || losetup -d $(< LOOP) || true
 		test ! -f LOOPFILE || rm -f $(< LOOPFILE)
 	fi
-	rm -f DEVICES # devs is set in prepare_devs()
+
 	not diff LOOP BACKING_DEV >/dev/null 2>&1 || rm -f BACKING_DEV
-	rm -f LOOP
+	rm -f DEVICES LOOP
 
 	# Attempt to remove any loop devices that failed to get torn down if earlier tests aborted
 	test "${LVM_TEST_PARALLEL:-0}" -eq 1 -o -z "$COMMON_PREFIX" || {
-		teardown_devs_prefixed "$COMMON_PREFIX" 1
 		local stray_loops=( $(losetup -a | grep "$COMMON_PREFIX" | cut -d: -f1) )
 		test ${#stray_loops[@]} -eq 0 || {
+			teardown_devs_prefixed "$COMMON_PREFIX" 1
 			echo "Removing stray loop devices containing $COMMON_PREFIX: ${stray_loops[@]}"
 			for i in "${stray_loops[@]}" ; do losetup -d $i ; done
+			# Leave test when udev processed all removed devices
+			udev_wait
 		}
 	}
-
-	# Leave test when udev processed all removed devices
-	udev_wait
 }
 
 kill_sleep_kill_() {
@@ -215,7 +225,7 @@ kill_sleep_kill_() {
 	slow=$2
 	if test -s $pidfile ; then
 		pid=$(< $pidfile)
-		kill -TERM $pid || return 0
+		kill -TERM $pid 2>/dev/null || return 0
 		if test $slow -eq 0 ; then sleep .1 ; else sleep 1 ; fi
 		kill -KILL $pid 2>/dev/null || true
 		wait=0
@@ -226,8 +236,46 @@ kill_sleep_kill_() {
 	fi
 }
 
+print_procs_by_tag_() {
+	(ps -o pid,args ehax | grep -we"LVM_TEST_TAG=${1:-kill_me_$PREFIX}") || true
+}
+
+count_processes_with_tag() {
+	print_procs_by_tag_ | wc -l
+}
+
+kill_tagged_processes() {
+	local pid
+	local pids
+	local wait
+
+	# read uses all vars within pipe subshell
+	print_procs_by_tag_ "$@" | while read -r pid wait; do
+		if test -n "$pid" ; then
+			echo "Killing tagged process: $pid ${wait:0:120}..."
+			kill -TERM $pid 2>/dev/null || true
+		fi
+		pids="$pids $pid"
+	done
+
+	# wait if process exited and eventually -KILL
+	wait=0
+	for pid in $pids ; do
+		while ps $pid > /dev/null && test $wait -le 10; do
+			sleep .2
+			wait=$(($wait + 1))
+		done
+		test $wait -le 10 || kill -KILL $pid 2>/dev/null || true
+	done
+}
+
 teardown() {
 	echo -n "## teardown..."
+	unset LVM_LOG_FILE_EPOCH
+
+	if test -f TESTNAME ; then
+
+	kill_tagged_processes
 
 	kill_sleep_kill_ LOCAL_LVMETAD ${LVM_VALGRIND_LVMETAD:-0}
 
@@ -250,6 +298,8 @@ teardown() {
 
 	echo -n .
 
+	fi
+
 	test -n "$TESTDIR" && {
 		cd "$TESTOLDPWD"
 		rm -rf "$TESTDIR" || echo BLA
@@ -284,7 +334,8 @@ prepare_loop() {
 	echo -n .
 
 	local LOOPFILE="$PWD/test.img"
-	dd if=/dev/zero of="$LOOPFILE" bs=$((1024*1024)) count=0 seek=$(($size)) 2> /dev/null
+	rm -f "$LOOPFILE"
+	dd if=/dev/zero of="$LOOPFILE" bs=$((1024*1024)) count=0 seek=$(($size + 1)) 2> /dev/null
 	if LOOP=$(losetup -s -f "$LOOPFILE" 2>/dev/null); then
 		:
 	elif LOOP=$(losetup -f) && losetup "$LOOP" "$LOOPFILE"; then
@@ -319,13 +370,11 @@ prepare_scsi_debug_dev() {
 	local DEV_SIZE=$1
 	local SCSI_DEBUG_PARAMS=${@:2}
 
+	rm -f debug.log strace.log
 	test ! -f "SCSI_DEBUG_DEV" || return 0
 	test -z "$LOOP"
 	test -n "$DM_DEV_DIR"
 
-	# Skip test if awk isn't available (required for get_sd_devs_)
-	which awk || skip
-
 	# Skip test if scsi_debug module is unavailable or is already in use
 	modprobe --dry-run scsi_debug || skip
 	lsmod | not grep -q scsi_debug || skip
@@ -333,6 +382,7 @@ prepare_scsi_debug_dev() {
 	# Create the scsi_debug device and determine the new scsi device's name
 	# NOTE: it will _never_ make sense to pass num_tgts param;
 	# last param wins.. so num_tgts=1 is imposed
+	touch SCSI_DEBUG_DEV
 	modprobe scsi_debug dev_size_mb=$DEV_SIZE $SCSI_DEBUG_PARAMS num_tgts=1 || skip
 	sleep 2 # allow for async Linux SCSI device registration
 
@@ -352,11 +402,74 @@ cleanup_scsi_debug_dev() {
 	rm -f SCSI_DEBUG_DEV LOOP
 }
 
+prepare_md_dev() {
+	local level=$1
+	local rchunk=$2
+	local rdevs=$3
+
+	local maj=$(mdadm --version 2>&1) || skip "mdadm tool is missing!"
+	local mddev
+
+	cleanup_md_dev
+
+	rm -f debug.log strace.log MD_DEV MD_DEV_PV MD_DEVICES
+
+	# Have MD use a non-standard name to avoid colliding with an existing MD device
+	# - mdadm >= 3.0 requires that non-standard device names be in /dev/md/
+	# - newer mdadm _completely_ defers to udev to create the associated device node
+	maj=${maj##*- v}
+	maj=${maj%%.*}
+	[ "$maj" -ge 3 ] && \
+		mddev=/dev/md/md_lvm_test0 || \
+		mddev=/dev/md_lvm_test0
+
+	mdadm --create --metadata=1.0 "$mddev" --auto=md --level $level --chunk $rchunk --raid-devices=$rdevs "${@:4}"
+	test -b "$mddev" || skip "mdadm has not created device!"
+
+	# LVM/DM will see this device
+	case "$DM_DEV_DIR" in
+	"/dev") readlink -f "$mddev" ;;
+	*)	cp -LR "$mddev" "$DM_DEV_DIR"
+		echo "$DM_DEV_DIR/md_lvm_test0" ;;
+	esac > MD_DEV_PV
+	echo "$mddev" > MD_DEV
+	notify_lvmetad $(< MD_DEV_PV)
+	printf "%s\n" "${@:4}" > MD_DEVICES
+	for mddev in "${@:4}"; do
+		notify_lvmetad "$mddev"
+	done
+}
+
+cleanup_md_dev() {
+	test -f MD_DEV || return 0
+
+	local IFS=$IFS_NL
+	local dev=$(< MD_DEV)
+
+	udev_wait
+	mdadm --stop "$dev" || true
+	test "$DM_DEV_DIR" != "/dev" && rm -f "$DM_DEV_DIR/$(basename $dev)"
+	notify_lvmetad $(< MD_DEV_PV)
+	for dev in $(< MD_DEVICES); do
+		mdadm --zero-superblock "$dev" || true
+		notify_lvmetad "$dev"
+	done
+	udev_wait
+	if [ -b "$mddev" ]; then
+		# mdadm doesn't always cleanup the device node
+		# sleeps offer hack to defeat: 'md: md127 still in use'
+		# see: https://bugzilla.redhat.com/show_bug.cgi?id=509908#c25
+		sleep 2
+		rm -f "$mddev"
+	fi
+	rm -f MD_DEV MD_DEVICES MD_DEV_PV
+}
+
 prepare_backing_dev() {
-	if test -f BACKING_DEV; then 
+	if test -f BACKING_DEV; then
 		BACKING_DEV=$(< BACKING_DEV)
 	elif test -b "$LVM_TEST_BACKING_DEVICE"; then
-		BACKING_DEV="$LVM_TEST_BACKING_DEVICE"
+		BACKING_DEV=$LVM_TEST_BACKING_DEVICE
 		echo "$BACKING_DEV" > BACKING_DEV
 	else
 		prepare_loop "$@"
@@ -367,8 +480,12 @@ prepare_devs() {
 	local n=${1:-3}
 	local devsize=${2:-34}
 	local pvname=${3:-pv}
+	local shift=0
 
+	touch DEVICES
 	prepare_backing_dev $(($n*$devsize))
+	# shift start of PV devices on /dev/loopXX by 1M
+	not diff LOOP BACKING_DEV >/dev/null 2>&1 || shift=2048
 	echo -n "## preparing $n devices..."
 
 	local size=$(($devsize*2048)) # sectors
@@ -379,7 +496,7 @@ prepare_devs() {
 		local dev="$DM_DEV_DIR/mapper/$name"
 		DEVICES[$count]=$dev
 		count=$(( $count + 1 ))
-		echo 0 $size linear "$BACKING_DEV" $((($i-1)*$size)) > "$name.table"
+		echo 0 $size linear "$BACKING_DEV" $((($i-1)*$size + $shift)) > "$name.table"
 		if not dmsetup create -u "TEST-$name" "$name" "$name.table" &&
 		   test -n "$LVM_TEST_BACKING_DEVICE";
 		then # maybe the backing device is too small for this test
@@ -393,7 +510,10 @@ prepare_devs() {
 
 	# non-ephemeral devices need to be cleared between tests
 	test -f LOOP || for d in ${DEVICES[@]}; do
-		dd if=/dev/zero of=$d bs=64K count=1
+		blkdiscard "$d" 2>/dev/null || true
+		# ensure disk header is always zeroed
+		dd if=/dev/zero of="$d" bs=4096 count=1
+		wipefs -a "$d" 2>/dev/null || dd if=/dev/zero of="$d" bs=64K count=1
 	done
 
 	#for i in `seq 1 $n`; do
@@ -408,18 +528,34 @@ prepare_devs() {
 	printf "%s\n" "${DEVICES[@]}" > DEVICES
 #	( IFS=$'\n'; echo "${DEVICES[*]}" ) >DEVICES
 	echo "ok"
+
+	for dev in "${DEVICES[@]}"; do
+		notify_lvmetad "$dev"
+	done
 }
 
-# Replace linear PV device with its 'delayed' version
-# Could be used to more deterministicaly hit some problems.
-# Parameters: {device path} [read delay ms] [write delay ms]
-# Original device is restored when both delay params are 0 (or missing).
-# i.e.  delay_dev "$dev1" 0 200
-delay_dev() {
-	target_at_least dm-delay 1 2 0 || skip
-	local name=$(echo "$1" | sed -e 's,.*/,,')
-	local read_ms=${2:-0}
-	local write_ms=${3:-0}
+
+common_dev_() {
+	local tgtype=$1
+	local name=${2##*/}
+	local offsets
+	local read_ms
+	local write_ms
+
+	case "$tgtype" in
+	delay)
+		read_ms=${3:-0}
+		write_ms=${4:-0}
+		offsets=${@:5}
+		if test "$read_ms" -eq 0 -a "$write_ms" -eq 0 ; then
+			offsets=
+		else
+			test -z "${offsets[@]}" && offsets="0:"
+		fi ;;
+	error)  offsets=${@:3}
+		test -z "${offsets[@]}" && offsets="0:" ;;
+	esac
+
 	local pos
 	local size
 	local type
@@ -428,17 +564,50 @@ delay_dev() {
 
 	read pos size type pvdev offset < "$name.table"
 
+	for fromlen in ${offsets[@]}; do
+		from=${fromlen%%:*}
+		len=${fromlen##*:}
+		test -n "$len" || len=$(($size - $from))
+		diff=$(($from - $pos))
+		if test $diff -gt 0 ; then
+			echo "$pos $diff $type $pvdev $(($pos + $offset))"
+			pos=$(($pos + $diff))
+		elif test $diff -lt 0 ; then
+			die "Position error"
+		fi
+
+		case "$tgtype" in
+		delay)
+			echo "$from $len delay $pvdev $(($pos + $offset)) $read_ms $pvdev $(($pos + $offset)) $write_ms" ;;
+		error)
+			echo "$from $len error" ;;
+		esac
+		pos=$(($pos + $len))
+	done > "$name.devtable"
+	diff=$(($size - $pos))
+	test "$diff" -gt 0 && echo "$pos $diff $type $pvdev $(($pos + $offset))" >>"$name.devtable"
+
 	init_udev_transaction
-	if test $read_ms -ne 0 -o $write_ms -ne 0 ; then
-		echo "0 $size delay $pvdev $offset $read_ms $pvdev $offset $write_ms" | \
-			dmsetup load "$name"
-	else
-		dmsetup load "$name" "$name.table"
-	fi
+	dmsetup load "$name" "$name.devtable"
+	# TODO: add support for resume without udev rescan
 	dmsetup resume "$name"
 	finish_udev_transaction
 }
 
+# Replace linear PV device with its 'delayed' version
+# Could be used to more deterministicaly hit some problems.
+# Parameters: {device path} [read delay ms] [write delay ms] [offset:size]...
+# Original device is restored when both delay params are 0 (or missing).
+# If the size is missing, the remaing portion of device is taken
+# i.e.  delay_dev "$dev1" 0 200 256:
+delay_dev() {
+	if test ! -f HAVE_DM_DELAY ; then
+		target_at_least dm-delay 1 1 0 || skip
+	fi
+	touch HAVE_DM_DELAY
+	common_dev_ delay "$@"
+}
+
 disable_dev() {
 	local dev
 	local silent
@@ -464,8 +633,8 @@ disable_dev() {
 		echo "Disabling device $dev ($maj:$min)"
 		notify="$notify $maj:$min"
 		if test -n "$error"; then
-		    echo 0 10000000 error | dmsetup load $dev
-		    dmsetup resume $dev
+		    echo 0 10000000 error | dmsetup load "$dev"
+		    dmsetup resume "$dev"
 		else
 		    dmsetup remove -f "$dev" 2>/dev/null || true
 		fi
@@ -508,46 +677,7 @@ enable_dev() {
 # Original device table is replace with multiple lines
 # i.e.  error_dev "$dev1" 8:32 96:8
 error_dev() {
-	local dev=$1
-	local name=$(echo "$dev" | sed -e 's,.*/,,')
-	local fromlen
-	local pos
-	local size
-	local type
-	local pvdev
-	local offset
-	local silent
-
-	read pos size type pvdev offset < $name.table
-
-	shift
-	rm -f $name.errtable
-	for fromlen in "$@"; do
-		from=${fromlen%%:*}
-		len=${fromlen##*:}
-		diff=$(($from - $pos))
-		if test $diff -gt 0 ; then
-			echo "$pos $diff $type $pvdev $(($pos + $offset))" >>$name.errtable
-			pos=$(($pos + $diff))
-		elif test $diff -lt 0 ; then
-			die "Position error"
-		fi
-		echo "$from $len error" >>$name.errtable
-		pos=$(($pos + $len))
-	done
-	diff=$(($size - $pos))
-	test $diff -gt 0 && echo "$pos $diff $type $pvdev $(($pos + $offset))" >>$name.errtable
-
-	init_udev_transaction
-	if dmsetup table $name ; then
-		dmsetup load "$name" "$name.errtable"
-	else
-		dmsetup create -u "TEST-$name" "$name" "$name.errtable"
-	fi
-	# using device name (since device path does not exists yet with udev)
-	dmsetup resume "$name"
-	finish_udev_transaction
-	test -n "$silent" || notify_lvmetad "$dev"
+	common_dev_ error "$@"
 }
 
 backup_dev() {
@@ -637,7 +767,7 @@ devices/scan = "$DM_DEV_DIR"
 devices/filter = "a|.*|"
 devices/global_filter = [ "a|$DM_DEV_DIR/mapper/.*pv[0-9_]*$|", "r|.*|" ]
 devices/cache_dir = "$TESTDIR/etc"
-devices/sysfs_scan = 0
+devices/sysfs_scan = 1
 devices/default_data_alignment = 1
 devices/md_component_detection  = 0
 log/syslog = 0
@@ -657,6 +787,12 @@ global/locking_dir = "$TESTDIR/var/lock/lvm"
 global/locking_type=$LVM_TEST_LOCKING
 global/si_unit_consistency = 1
 global/fallback_to_local_locking = 0
+global/cache_check_executable = "$LVM_TEST_CACHE_CHECK_CMD"
+global/cache_dump_executable = "$LVM_TEST_CACHE_DUMP_CMD"
+global/cache_repair_executable = "$LVM_TEST_CACHE_REPAIR_CMD"
+global/thin_check_executable = "$LVM_TEST_THIN_CHECK_CMD"
+global/thin_dump_executable = "$LVM_TEST_THIN_DUMP_CMD"
+global/thin_repair_executable = "$LVM_TEST_THIN_REPAIR_CMD"
 activation/checks = 1
 activation/udev_sync = 1
 activation/udev_rules = 1
@@ -674,16 +810,43 @@ EOF
 	    echo "$v"
 	done >> "$config_values"
 
-	local s
-	for s in $(cut -f1 -d/ "$config_values" | sort | uniq); do
-		echo "$s {"
-		local k
-		for k in $(grep ^"$s"/ "$config_values" | cut -f1 -d= | sed -e 's, *$,,' | sort | uniq); do
-			grep "^$k" "$config_values" | tail -n 1 | sed -e "s,^$s/,	  ,"
-		done
-		echo "}"
-		echo
-	done | tee "$config"
+	declare -A CONF 2>/dev/null || {
+		# Associative arrays is not available
+		local s
+		for s in $(cut -f1 -d/ "$config_values" | sort | uniq); do
+			echo "$s {"
+			local k
+			for k in $(grep ^"$s"/ "$config_values" | cut -f1 -d= | sed -e 's, *$,,' | sort | uniq); do
+				grep "^$k" "$config_values" | tail -n 1 | sed -e "s,^$s/,	 ,"
+			done
+			echo "}"
+			echo
+		done | tee "$config" | sed -e "s,^,## LVMCONF: ,"
+		return 0
+	}
+
+	local sec
+	local last_sec
+
+	# read sequential list and put into associative array
+	while IFS=$IFS_NL read -r v; do
+		# trim white-space-chars via echo when inserting
+		CONF[$(echo ${v%%[={]*})]=${v#*/}
+	done < "$config_values"
+
+	# sort by section and iterate through them
+	printf "%s\n" ${!CONF[@]} | sort | while read -r v ; do
+		sec=${v%%/*} # split on section'/'param_name
+		test "$sec" = "$last_sec" || {
+			test -z "$last_sec" || echo "}"
+			echo "$sec {"
+			last_sec=$sec
+		}
+		echo "    ${CONF[$v]}"
+	done > "$config"
+	echo "}" >> "$config"
+
+	sed -e "s,^,## LVMCONF: ," "$config"
 }
 
 lvmconf() {
@@ -696,28 +859,21 @@ profileconf() {
 	profile_name="$1"
 	shift
 	generate_config "$@"
-	test -d etc/profile || mkdir etc/profile
+	mkdir -p etc/profile
 	mv -f "PROFILE_$profile_name" "etc/profile/$profile_name.profile"
 }
 
 prepare_profiles() {
-	test -d etc/profile || mkdir etc/profile
+	mkdir -p etc/profile
 	for profile_name in $@; do
-		test -L "$abs_top_builddir/test/lib/$profile_name.profile" || skip
-		cp "$abs_top_builddir/test/lib/$profile_name.profile" "etc/profile/$profile_name.profile"
+		test -L "lib/$profile_name.profile" || skip
+		cp "lib/$profile_name.profile" "etc/profile/$profile_name.profile"
 	done
 }
 
 apitest() {
-	local t=$1
-	shift
-	test -x "$abs_top_builddir/test/api/$t.t" || skip
-	"$abs_top_builddir/test/api/$t.t" "$@" && rm -f debug.log strace.log
-}
-
-api() {
-	test -x "$abs_top_builddir/test/api/wrapper" || skip
-	"$abs_top_builddir/test/api/wrapper" "$@" && rm -f debug.log strace.log
+	test -x "$TESTOLDPWD/api/$1.t" || skip
+	"$TESTOLDPWD/api/$1.t" "${@:2}" && rm -f debug.log strace.log
 }
 
 mirror_recovery_works() {
@@ -804,7 +960,7 @@ version_at_least() {
 	local major
 	local minor
 	local revision
-	IFS=. read -r major minor revision <<< "$1"
+	IFS=".-" read -r major minor revision <<< "$1"
 	shift
 
 	test -z "$1" && return 0
@@ -841,39 +997,67 @@ target_at_least() {
 
 	local version=$(dmsetup targets 2>/dev/null | grep "${1##dm-} " 2>/dev/null)
 	version=${version##* v}
-	shift
 
-	version_at_least "$version" "$@"
+	version_at_least "$version" "${@:2}" || {
+		echo "Found $1 version $version, but requested ${*:2}." >&2
+		return 1
+	}
 }
 
 have_thin() {
-	test "$THIN" = shared -o "$THIN" = internal || return 1
-	target_at_least dm-thin-pool "$@" || return 1
+	test "$THIN" = shared -o "$THIN" = internal || {
+		echo "Thin is not built-in." >&2
+		return 1;
+	}
+	target_at_least dm-thin-pool "$@"
 
+	declare -a CONF
 	# disable thin_check if not present in system
-	test -x "$LVM_TEST_THIN_CHECK_CMD" || LVM_TEST_THIN_CHECK_CMD=""
-	test -x "$LVM_TEST_THIN_DUMP_CMD" || LVM_TEST_THIN_DUMP_CMD=""
-	test -x "$LVM_TEST_THIN_REPAIR_CMD" || LVM_TEST_THIN_REPAIR_CMD=""
-	lvmconf "global/thin_check_executable = \"$LVM_TEST_THIN_CHECK_CMD\"" \
-		"global/thin_dump_executable = \"$LVM_TEST_THIN_DUMP_CMD\"" \
-		"global/thin_repair_executable = \"$LVM_TEST_THIN_REPAIR_CMD\""
+	if test -n "$LVM_TEST_THIN_CHECK_CMD" -a ! -x "$LVM_TEST_THIN_CHECK_CMD" ; then
+		CONF[0]="global/thin_check_executable = \"\""
+	fi
+	if test -n "$LVM_TEST_THIN_DUMP_CMD" -a ! -x "$LVM_TEST_THIN_DUMP_CMD" ; then
+		CONF[1]="global/thin_dump_executable = \"\""
+	fi
+	if test -n "$LVM_TEST_THIN_REPAIR_CMD" -a ! -x "$LVM_TEST_THIN_REPAIR_CMD" ; then
+		CONF[2]="global/thin_repair_executable = \"\""
+	fi
+	if test ${#CONF[@]} -ne 0 ; then
+		echo "TEST WARNING: Reconfiguring ${CONF[@]}"
+		lvmconf "${CONF[@]}"
+	fi
 }
 
 have_raid() {
-	test "$RAID" = shared -o "$RAID" = internal || return 1
+	test "$RAID" = shared -o "$RAID" = internal || {
+		echo "Raid is not built-in." >&2
+		return 1;
+	}
 	target_at_least dm-raid "$@"
 }
 
 have_cache() {
-	test "$CACHE" = shared -o "$CACHE" = internal || return 1
+	test "$CACHE" = shared -o "$CACHE" = internal || {
+		echo "Cache is not built-in." >&2
+		return 1;
+	}
 	target_at_least dm-cache "$@"
 
-	test -x "$LVM_TEST_CACHE_CHECK_CMD" || LVM_TEST_CACHE_CHECK_CMD=""
-	test -x "$LVM_TEST_CACHE_DUMP_CMD" || LVM_TEST_CACHE_DUMP_CMD=""
-	test -x "$LVM_TEST_CACHE_REPAIR_CMD" || LVM_TEST_CACHE_REPAIR_CMD=""
-	lvmconf "global/cache_check_executable = \"$LVM_TEST_CACHE_CHECK_CMD\"" \
-		"global/cache_dump_executable = \"$LVM_TEST_CACHE_DUMP_CMD\"" \
-		"global/cache_repair_executable = \"$LVM_TEST_CACHE_REPAIR_CMD\""
+	declare -a CONF
+	# disable cache_check if not present in system
+	if test -n "$LVM_TEST_CACHE_CHECK_CMD" -a ! -x "$LVM_TEST_CACHE_CHECK_CMD" ; then
+		CONF[0]="global/cache_check_executable = \"\""
+	fi
+	if test -n "$LVM_TEST_CACHE_DUMP_CMD" -a ! -x "$LVM_TEST_CACHE_DUMP_CMD" ; then
+		CONF[1]="global/cache_dump_executable = \"\""
+	fi
+	if test -n "$LVM_TEST_CACHE_REPAIR_CMD" -a ! -x "$LVM_TEST_CACHE_REPAIR_CMD" ; then
+		CONF[2]="global/cache_repair_executable = \"\""
+	fi
+	if test ${#CONF[@]} -ne 0 ; then
+		echo "TEST WARNING: Reconfiguring ${CONF[@]}"
+		lvmconf "${CONF[@]}"
+	fi
 }
 
 have_tool_at_least() {
@@ -894,6 +1078,31 @@ dmsetup_wrapped() {
 	dmsetup "$@"
 }
 
+wait_pvmove_lv_ready() {
+	# given sleep .1 this is about 60 secs of waiting
+	local retries=${2:-300}
+	while : ; do
+		test $retries -le 0 && die "Waiting for pvmove LV to get activated has timed out"
+		dmsetup info -c -o tables_loaded $1 > out 2>/dev/null|| true;
+		not grep Live out >/dev/null || break
+		sleep .1
+		retries=$((retries-1))
+	done
+}
+
+# return total memory size in kB units
+total_mem() {
+	while IFS=":" read -r a b ; do
+		case "$a" in MemTotal*) echo ${b%% kB} ; break ;; esac
+	done < /proc/meminfo
+}
+
+kernel_at_least() {
+	version_at_least "$(uname -r)" "$@"
+}
+
+test -z "$LVM_TEST_AUX_TRACE" || set -x
+
 test -f DEVICES && devs=$(< DEVICES)
 
 if test "$1" = dmsetup; then
