@@ -296,6 +296,7 @@ struct TimedBuffer {
 
     std::deque< Line > data;
     Line incomplete;
+    bool stamp;
 
     Line shift( bool force = false ) {
         Line result = std::make_pair( 0, "" );
@@ -309,7 +310,7 @@ struct TimedBuffer {
     }
 
     void push( std::string buf ) {
-        time_t now = time( 0 );
+        time_t now = stamp ? time( 0 ) : 0;
         std::string::iterator b = buf.begin(), e = buf.begin();
 
         while ( e != buf.end() )
@@ -323,6 +324,18 @@ struct TimedBuffer {
             if ( e != buf.end() ) {
                 incomplete.second += "\n";
                 data.push_back( incomplete );
+                if (incomplete.second[0] == '#') {
+                    /* Disable timing between '## 0 STACKTRACE' & '## teardown' keywords */
+                    if (incomplete.second.find("# 0 STACKTRACE", 1) != std::string::npos ||
+                        incomplete.second.find("# timing off", 1) != std::string::npos) {
+                        stamp = false;
+                        now = 0;
+                    } else if (incomplete.second.find("# teardown", 1) != std::string::npos ||
+                               incomplete.second.find("# timing on", 1) != std::string::npos) {
+                        stamp = true;
+                        now = time( 0 );
+                    }
+                }
                 incomplete = std::make_pair( now, "" );
             }
             b = (e == buf.end() ? e : e + 1);
@@ -334,6 +347,8 @@ struct TimedBuffer {
             return false;
         return data.empty();
     }
+
+    TimedBuffer() : stamp(true) {}
 };
 
 struct Sink {
@@ -345,18 +360,23 @@ struct Sink {
 
 struct Substitute {
     typedef std::map< std::string, std::string > Map;
-    Map _map;
+    std::string testdir; // replace testdir first
+    std::string prefix;
 
     std::string map( std::string line ) {
         if ( std::string( line, 0, 9 ) == "@TESTDIR=" )
-            _map[ "@TESTDIR@" ] = std::string( line, 9, std::string::npos );
+            testdir = std::string( line, 9, line.length() - 10 ); // skip \n
         else if ( std::string( line, 0, 8 ) == "@PREFIX=" )
-            _map[ "@PREFIX@" ] = std::string( line, 8, std::string::npos );
+            prefix = std::string( line, 8, line.length() - 9 );  // skip \n
         else {
             size_t off;
-            for ( Map::iterator s = _map.begin(); s != _map.end(); ++s )
-                while ( (off = line.find( s->first )) != std::string::npos )
-                    line.replace( off, s->first.length(), s->second );
+            if (!testdir.empty())
+                while ( (off = line.find( testdir )) != std::string::npos )
+                    line.replace( off, testdir.length(), "@TESTDIR@" );
+
+            if (!prefix.empty())
+                while ( (off = line.find( prefix )) != std::string::npos )
+                    line.replace( off, prefix.length(), "@PREFIX@" );
         }
         return line;
     }
@@ -364,12 +384,11 @@ struct Substitute {
 
 struct Format {
     time_t start;
-    bool stamp;
     Substitute subst;
 
     std::string format( TimedBuffer::Line l ) {
         std::stringstream result;
-        if ( stamp ) {
+        if ( l.first >= start ) {
             time_t rel = l.first - start;
             result << "[" << std::setw( 2 ) << std::setfill( ' ' ) << rel / 60
                    << ":" << std::setw( 2 ) << std::setfill( '0' ) << rel % 60 << "] ";
@@ -378,7 +397,7 @@ struct Format {
         return result.str();
     }
 
-    Format() : start( time( 0 ) ), stamp( true ) {}
+    Format() : start( time( 0 ) ) {}
 };
 
 struct BufSink : Sink {
@@ -904,8 +923,9 @@ struct TestCase {
 
         if ( options.batch ) {
             int spaces = std::max( 64 - int(pretty().length()), 0 );
-            progress( Last ) << " " << std::string( spaces, '.' ) << " " << r;
-            if ( r == Journal::PASSED )
+            progress( Last ) << " " << std::string( spaces, '.' ) << " "
+                << std::left << std::setw( 9 ) << std::setfill( ' ' ) << r;
+            if ( r != Journal::SKIPPED )
                 progress( First ) << "   " << rusage();
             progress( Last ) << std::endl;
         } else
