@@ -32,9 +32,11 @@ PREFIX="${COMMON_PREFIX}$$"
 if test -z "$LVM_TEST_DIR"; then LVM_TEST_DIR=$TMPDIR; fi
 TESTDIR=$(mkdtemp "${LVM_TEST_DIR:-/tmp}" "$PREFIX.XXXXXXXXXX") || \
 	die "failed to create temporary directory in ${LVM_TEST_DIR:-$TESTOLDPWD}"
-RUNNING_DMEVENTD=$(pgrep dmeventd) || true
+RUNNING_DMEVENTD=$(pgrep dmeventd || true)
 
 export TESTOLDPWD TESTDIR COMMON_PREFIX PREFIX RUNNING_DMEVENTD
+export LVM_LOG_FILE_EPOCH=DEBUG
+export LVM_EXPECTED_EXIT_STATUS=1
 
 test -n "$BASH" && trap 'set +vx; STACKTRACE; set -vx' ERR
 trap 'aux teardown' EXIT # don't forget to clean up
@@ -60,7 +62,7 @@ mkdir "$LVM_SYSTEM_DIR" "$DM_DEV_DIR"
 if test -n "$LVM_TEST_DEVDIR" ; then
 	DM_DEV_DIR=$LVM_TEST_DEVDIR
 else
-	mknod "$DM_DEV_DIR/testnull" c 1 3 || die "mknod failed";
+	mknod "$DM_DEV_DIR/testnull" c 1 3 || die "mknod failed"
 	echo >"$DM_DEV_DIR/testnull" || \
 		die "Filesystem does support devices in $DM_DEV_DIR (mounted with nodev?)"
 	mkdir "$DM_DEV_DIR/mapper"
@@ -75,27 +77,28 @@ echo "$TESTNAME" >TESTNAME
 
 echo "Kernel is $(uname -a)"
 # Report SELinux mode
-if which getenforce &>/dev/null ; then
-	echo "Selinux mode is \"$(getenforce 2>/dev/null)\"."
-else
-	echo "Selinux mode is not installed."
-fi
+echo "Selinux mode is $(getenforce 2>/dev/null || echo not installed)."
+free -m || true
 
 # Set vars from utils now that we have TESTDIR/PREFIX/...
 prepare_test_vars
 
 test -n "$BASH" && set -eE -o pipefail
 
-aux lvmconf
-aux prepare_clvmd
-test -n "$LVM_TEST_LVMETAD" && {
-	export LVM_LVMETAD_SOCKET="$TESTDIR/lvmetad.socket"
-	export LVM_LVMETAD_PIDFILE="$TESTDIR/lvmetad.pid"
-	aux prepare_lvmetad
-}
-
 # Vars for harness
 echo "@TESTDIR=$TESTDIR"
 echo "@PREFIX=$PREFIX"
 
+if test -n "$LVM_TEST_LVMETAD" ; then
+	export LVM_LVMETAD_SOCKET="$TESTDIR/lvmetad.socket"
+	export LVM_LVMETAD_PIDFILE="$TESTDIR/lvmetad.pid"
+	aux prepare_lvmetad
+else
+	# lvmetad prepares its own lvmconf
+	aux lvmconf
+	aux prepare_clvmd
+fi
+
+echo "<======== Processing test: \"$TESTNAME\" ========>"
+
 set -vx
