@@ -12,10 +12,10 @@
 
 test_description="ensure that 'vgreduce --removemissing' works on mirrored LV"
 
-. lib/inittest
-
 # disable lvmetad logging as it bogs down test systems
-test -e LOCAL_LVMETAD && aux prepare_lvmetad ""
+export LVM_TEST_LVMETAD_DEBUG_OPTS=${LVM_TEST_LVMETAD_DEBUG_OPTS-}
+
+. lib/inittest
 
 lv_is_on_ ()
 {
