@@ -11,10 +11,10 @@
 
 test_description='Exercise toollib process_each_lv'
 
-. lib/inittest
-
 # disable lvmetad logging as it bogs down test systems
-test -e LOCAL_LVMETAD && aux prepare_lvmetad ""
+export LVM_TEST_LVMETAD_DEBUG_OPTS=${LVM_TEST_LVMETAD_DEBUG_OPTS-}
+
+. lib/inittest
 
 aux prepare_devs 10
 
