@@ -16,6 +16,6 @@ pvremove "$dev1" "$dev2" 2>&1 | tee pvremove.txt
 not grep "No physical" pvremove.txt
 
 pvcreate "$dev1" "$dev2"
-vgcreate bla $dev1 $dev2
-pvremove -ff -y $dev1 $dev2 2>&1 | tee pvremove.txt
+vgcreate bla "$dev1" "$dev2"
+pvremove -ff -y "$dev1" "$dev2" 2>&1 | tee pvremove.txt
 not grep "device missing" pvremove.txt
