@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright (C) 2008-2013 Red Hat, Inc. All rights reserved.
+# Copyright (C) 2015 Red Hat, Inc. All rights reserved.
 #
 # This copyrighted material is made available to anyone wishing to use,
 # modify, copy, or redistribute it subject to the terms and conditions
@@ -15,6 +15,26 @@ test_description='Test system_id'
 
 aux prepare_devs 1
 
+# with clvm enabled, vgcreate with no -c option creates a clustered vg,
+# which should have no system id
+
+if [ -e LOCAL_CLVMD ]; then
+SID1=sidfoolocal
+SID2=""
+LVMLOCAL=etc/lvmlocal.conf
+rm -f $LVMLOCAL
+echo "local {" > $LVMLOCAL
+echo "  system_id = $SID1" >> $LVMLOCAL
+echo "}" >> $LVMLOCAL
+aux lvmconf "global/system_id_source = lvmlocal"
+vgcreate $vg1 "$dev1"
+vgs -o+systemid $vg1
+check vg_field $vg1 systemid $SID2
+vgremove $vg1
+rm -f $LVMLOCAL
+exit 0
+fi
+
 # create vg with system_id using each source
 
 ## none
@@ -69,8 +89,8 @@ SID=sidfoofile
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 vgcreate $vg1 "$dev1"
 vgs -o+systemid $vg1
 check vg_field $vg1 systemid $SID
@@ -84,8 +104,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg, overriding the local system_id so the vg looks foreign
 vgcreate --systemid $SID2 $vg1 "$dev1"
 # normal vgs is not an error and does not see the vg
@@ -116,8 +136,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -145,8 +165,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -176,8 +196,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 lvcreate -n $lv1 -l 2 $vg1
@@ -209,8 +229,8 @@ SID1=sidfoofile1
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 aux lvmconf "global/system_id_source = none"
@@ -235,8 +255,8 @@ vgcreate $vg1 "$dev1"
 check vg_field $vg1 systemid ""
 # set a local system_id
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # check we can see and use the vg with no system_id
 vgs >err
 grep $vg1 err
@@ -251,8 +271,8 @@ SID1=sidfoofile1
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -278,8 +298,8 @@ SID1=sidfoofile1
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -312,8 +332,8 @@ SID2=012345678901234567890123456789012345678901234567890123456789012345678901234
 # max len system_id should appear normally
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -326,8 +346,8 @@ rm -f $SIDFILE
 # max+1 len system_id should be missing the last character
 rm -f $SIDFILE
 echo "$SID2" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -347,8 +367,8 @@ SID1=012345678901234567890123456789012345678901234567890123456789012345678901234
 SID2=012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789abcdefg
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -366,8 +386,8 @@ SID2=A.1
 
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -387,8 +407,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal pvs sees the vg and pv
@@ -418,8 +438,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 lvcreate -n $lv1 -l 2 $vg1
@@ -453,8 +473,8 @@ LVMLOCAL=etc/lvmlocal.conf
 rm -f $LVMLOCAL
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -483,8 +503,8 @@ SID1=sidfoofile1
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate --systemid "" $vg1 "$dev1"
 # normal vgs sees the vg
@@ -501,8 +521,8 @@ SID1=sidfoofile1
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -529,8 +549,8 @@ SIDFILE=etc/lvm_test.conf
 rm -f $LVMLOCAL
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -560,8 +580,8 @@ SIDFILE=etc/lvm_test.conf
 rm -f $LVMLOCAL
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg
 vgcreate $vg1 "$dev1"
 # normal vgs sees the vg
@@ -657,8 +677,8 @@ rm -f $SIDFILE
 SID=""
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 vgcreate $vg1 "$dev1" |& tee err
 vgs -o+systemid $vg1
 check vg_field $vg1 systemid $SID
@@ -680,8 +700,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg with an lv
 vgcreate $vg1 "$dev1"
 lvcreate -n $lv1 -l 2 -an $vg1
@@ -721,8 +741,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg with an lv
 vgcreate $vg1 "$dev1"
 lvcreate -n $lv1 -l 2 -an $vg1
@@ -761,8 +781,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg with an lv
 vgcreate $vg1 "$dev1"
 lvcreate -n $lv1 -l 2 -an $vg1
@@ -813,8 +833,8 @@ SID2=sidfoofile2
 SIDFILE=etc/lvm_test.conf
 rm -f $SIDFILE
 echo "$SID1" > $SIDFILE
-aux lvmconf "global/system_id_source = file"
-aux lvmconf "global/system_id_file = $SIDFILE"
+aux lvmconf "global/system_id_source = file" \
+	    "global/system_id_file = \"$SIDFILE\""
 # create a vg with an lv
 vgcreate $vg1 "$dev1"
 lvcreate -n $lv1 -l 2 -an $vg1
