@@ -11,10 +11,10 @@
 
 # Test vgsplit operation, including different LV types
 
-. lib/inittest
-
 # disable lvmetad logging as it bogs down test systems
-test -e LOCAL_LVMETAD && aux prepare_lvmetad ""
+export LVM_TEST_LVMETAD_DEBUG_OPTS=${LVM_TEST_LVMETAD_DEBUG_OPTS-}
+
+. lib/inittest
 
 COMM() {
 	LAST_TEST="$@"
@@ -119,9 +119,12 @@ COMM "vgsplit correctly splits mirror (log+leg on same dev) into $i VG ($j args)
 		lvremove -f $vg2/$lv1
 		vgremove -f $vg1 $vg2
 
+# Can't use mirrored log without cmirrord
+# TODO: Should work for inactive device, needs some fixes....
+if test ! -e LOCAL_CLVMD ; then
 COMM "vgsplit correctly splits mirror LV with mirrored log into $i VG ($j args)"
-		create_vg_ -c n $vg1 "$dev1" "$dev2" "$dev3" "$dev4"
-		test $i = existing && create_vg_ -c n $vg2 "$dev5"
+		create_vg_ $vg1 "$dev1" "$dev2" "$dev3" "$dev4"
+		test $i = existing && create_vg_ $vg2 "$dev5"
 
 		lvcreate -an -Zn -l 64 --mirrorlog mirrored --type mirror -m1 -n $lv1 $vg1 \
 		    "$dev1" "$dev2" "$dev3" "$dev4"
@@ -146,8 +149,8 @@ COMM "vgsplit correctly splits mirror LV with mirrored log into $i VG ($j args)"
 
 # RHBZ 875903
 COMM "vgsplit correctly splits mirror LV with mirrored log on same devs into $i VG ($j args)"
-		create_vg_ -c n $vg1 "$dev1" "$dev2" "$dev3" "$dev4"
-		test $i = existing && create_vg_ -c n $vg2 "$dev5"
+		create_vg_ $vg1 "$dev1" "$dev2" "$dev3" "$dev4"
+		test $i = existing && create_vg_ $vg2 "$dev5"
 
 		lvcreate -an -Zn -l 64 --mirrorlog mirrored --type mirror -m1 -n $lv1 $vg1 \
 		    "$dev1" "$dev2"
@@ -166,6 +169,7 @@ COMM "vgsplit correctly splits mirror LV with mirrored log on same devs into $i
 		fi
 		lvremove -f $vg2/$lv1
 		vgremove -f $vg1 $vg2
+fi
 
 COMM "vgsplit correctly splits origin and snapshot LV into $i VG ($j args)"
 		create_vg_ $vg1 "$dev1" "$dev2"
